@@ -2,9 +2,7 @@
 #'
 #' Calculates the surprisal (in bits) that a cumulative distribution function
 #' probability is at least that extreme.
-#'
-#' Useful for treating the cdf probabilities as residuals
-#' particularly when directional = TRUE.
+#' `r lifecycle::badge('deprecated')`
 #'
 #' @param x A numeric vector of values between 0 and 1.
 #' @inheritParams params
@@ -16,6 +14,7 @@
 #' sextreme(seq(0.1, 0.9, by = 0.1))
 #' sextreme(seq(0.1, 0.9, by = 0.1), directional = TRUE)
 sextreme <- function(x, directional = FALSE) {
+  lifecycle::deprecate_soft("0.1.1", "sextreme()", id = "sextreme")
   chk_flag(directional)
   if(!length(x)) return(numeric(0))
   s <- -log(pextreme(x), 2)
