@@ -12,15 +12,17 @@
 #' @export
 #' @examples
 #' upper(as.numeric(0:100))
-upper <- function(x, conf_level = 0.95) {
+upper <- function(x, conf_level = 0.95, na_rm = FALSE) {
   chk_numeric(x)
   chk_number(conf_level)
   chk_range(conf_level)
 
-  if (!length(x)) {
-    return(NA_real_)
+  if(anyNA(x)) {
+    if(vld_false(na_rm)) return(NA_real_)
+    x <- as.vector(x)
+    x <- x[!is.na(x)]
   }
-  if (anyNA(x)) {
+  if (!length(x)) {
     return(NA_real_)
   }
 
