@@ -12,6 +12,7 @@ test_that("translations", {
   expect_equal(x, 0.75)
   expect_equal(ilogit(logit(0.67)), 0.67)
   expect_equal(invlogit(logit(0.67)), 0.67)
+  expect_equal(inv_logit(logit(0.67)), 0.67)
 })
 
 test_that("translations2", {
@@ -19,6 +20,7 @@ test_that("translations2", {
   expect_identical(logit(x), qlogis(x))
   expect_identical(ilogit(logit(x)), x)
   expect_identical(invlogit(logit(x)), x)
+  expect_identical(inv_logit(logit(x)), x)
 
   logit(x) <- c(0.5, 1)
   expect_identical(x, ilogit(c(0.5, 1)))
